from loader import Loader
from towns import TownDB

import wx
import wx.lib.dialogs as dialogs

class MapView(wx.Frame):

	_sizeX = 3
	_sizeY = 2
	_tileSize = 256

	cursor = {"z": 0, "x": 0, "y": 0}

	def __init__(self, *args, **kwargs):
		super(self.__class__, self).__init__(*args, **kwargs)
		
		# get server URL
		server = dialogs.textEntryDialog(self, message="Please input server's address (IP:PORT):", title="Server's address", defaultText="192.168.2.76:80", style=wx.OK)
		self.loader = Loader(server.text)

		self.townDB = None
		self.touristDB = None

		self.Bind(wx.EVT_PAINT, self.OnPaint)
		self.Bind(wx.EVT_MOUSEWHEEL, self.OnScroll)
		self.Bind(wx.EVT_MOTION, self.OnMotion)

		self.InitMenu()

		self.Show()

	def InitMenu(self):
		m = wx.MenuBar()
		self.SetMenuBar(m)
		act = wx.Menu()

		gotoP = act.Append(wx.ID_ANY, "Go to point...")
		self.Bind(wx.EVT_MENU, self.GotoPoint, gotoP)

		gotoT= act.Append(wx.ID_ANY, "Go to town...")
		self.Bind(wx.EVT_MENU, self.GotoTown, gotoT)

		showT = act.Append(wx.ID_ANY, "Show tourist paths")
		self.Bind(wx.EVT_MENU, self.ShowTourist, showT)

		m.Append(act, "&Actions")

	def GotoPoint(self, e):
		newCursor = dict()
		for one in "z", "x", "y":
			res = dialogs.textEntryDialog(self, message="Please input '%s' coordinate:" % one, title="Coordinate input", defaultText="0", style=wx.OK)
			try:
				val = int(res.text)
				newCursor[one] = val
			except ValueError: # not an int
				wx.LogWarning("Invalid number.")
				return
		limits = self.GetLimits(newCursor["z"])
		if newCursor["x"] > limits["x"] or newCursor["y"] > limits["y"]:
			wx.LogWarning("Given coordinates doesn't exists.")
			return
		self.cursor = newCursor
		self.Refresh()

	def GotoTown(self, e):
		newCursor = dict()

		if not self.townDB:
			self.townDB = TownDB(self.loader)
			self.townDB.LoadFromXml()

		# get town
		towns = [unicode(town) for town in self.townDB]
		res = dialogs.singleChoiceDialog(self, message="Please select a town:", title="Town selection", lst=towns)
		if not res.accepted:
			return

		# get town object from response
		for one in self.townDB:
			if unicode(one) == res.selection:
				town = one
				break

		# get zoom
		res = dialogs.textEntryDialog(self, message="Please input '%s' coordinate:" % "z", title="Coordinate input", defaultText="3", style=wx.OK)
		zoom = 0
		try:
			zoom = int(res.text)
		except ValueError: # not an int
			wx.LogWarning("Invalid number.")
			return

		tile = self.XYtoTile(town.x, town.y)
		self.cursor = self.TransposeCoordinates(9, tile["x"], tile["y"], zNew=zoom) # 9 = max zoom
		print self.cursor

		self.Refresh()

	def ShowTourist(self, e):


		if not self.touristDB:
			self.touristDB = TouristDB(self.loader)
			self.touristDB.LoadFromXml()

		self.Refresh()


	def OnScroll(self, e):
		"""Mouse wheel handler."""
		zOld = self.cursor["z"]

		if e.GetWheelRotation() > 0 and zOld < 9: # 9 = max zoom
			zNew = zOld + 1
		elif e.GetWheelRotation() < 0 and zOld > 0:
			zNew = zOld - 1

		if zNew:
			self.cursor = self.TransposeCoordinates(zOld, self.cursor["x"], self.cursor["y"], zNew)
			self.Refresh()


	def OnMotion(self, e):
		"""Mouse motion handler."""
		if not e.LeftIsDown():
			return

		# TODO

		self.Refresh()


	def OnPaint(self, e):
		"""Paint handler."""
		dc = wx.PaintDC(self)
		w, h = self.GetClientSize()
		# draw siblings from cursor
		for sibling in self.GetSiblings(**self.cursor):
			tile = self.loader.LoadTile(self.cursor["z"], sibling["x"], sibling["y"])
			# offset to top left
			offset = self.GetOffset()
			x, y = sibling["x"], sibling["y"]
			x -= offset["x"]
			y -= offset["y"]
			if tile:
				bmp = wx.ImageFromStream(tile, wx.BITMAP_TYPE_JPEG).ConvertToBitmap()
				dc.DrawBitmap(bmp, x*self._tileSize, y*self._tileSize)


	def GetOffset(self):
		"""Returns an offset to use by painter."""
		limits = self.GetLimits()
		if self.cursor["x"] == 0 or self.cursor["y"] == 0 or self.cursor["x"] == limits["x"] or self.cursor["y"] == limits["y"]:
			return {"x":self.cursor["x"], "y":self.cursor["y"]}
		else:
			return {"x":self.cursor["x"]-1, "y":self.cursor["y"]-1}

	def GetLimits(self, z=None):
		if z:
			return {"x":self._sizeX * 2**z, "y":self._sizeY * 2**z}
		else:
			return {"x":self._sizeX * 2**self.cursor["z"], "y":self._sizeY * 2**self.cursor["z"]}

	def GetSiblings(self, z, x, y):
		"""Returns siblings (1 each way) for a tile."""
		limits = self.GetLimits(z)
		return [{"x":a, "y":b} for a in xrange(x-1, x+2) for b in xrange(y-1, y+2)
			if a >= 0 and b >= 0
			and a < limits["x"] and b < limits["y"]]

	def TransposeCoordinates(self, zOld, x, y, zNew):
		"""Returns coordinates transposed from one zoom level to other."""
		# TODO edit
		return {"z":zNew, "x":int(2**(zNew-zOld)*x), "y":int(2**(zNew-zOld)*y)}

	def XYtoTile(self, x, y):
		return {"x":x/self._tileSize, "y":y/self._tileSize}


if __name__ == "__main__":
	a = wx.App(None)
	m = MapView(None, wx.ID_ANY, "Map View")
	#m.GotoTown(None)
	a.MainLoop()
	